<?php
require_once 'aibridge.class.php';

class LogSaver
{
  function outputSaveableDiagnosisLog($forAllLog, &$html)
  {
    // Call AI for diagnosis log
    $stdout = AIBridge::getDiagnosisLog();

    // Try to make stdout into a XMLDocument
    $xml = new XMLDocument($stdout);
    if ($xml)
    {
      $document = $xml->document_element();
      if ($document->tagname() == 'error') // If AI returned an error:
      {
        $outputArray = array(
          'name'  => 'AI Error',
          'value' => $document->get_content());
        if ($forAllLog == false) die(Rosetta::arrayToJSON($outputArray));
      }
      $xpath = xpath_new_context($document);
      $records = @ xpath_eval($xpath, '/diagnosisLogRecord');
    }
    else
    {
      $outputArray = array(
        'name'  => 'XML Error',
        'value' => $stdout);
      if ($forAllLog == false) die(Rosetta::arrayToJSON($outputArray));
    }

    if (count($records->nodeset) > 0)
    {
      $html[] = '<table class="i">';
      $html[] = '<tr>';
      $html[] = ('<td class="p"><span>' . translate('MSG_PASSED') . '</span></td>');
      $html[] = ('<td> - ' . translate('MSG_DENOTES_PASSED_TESTS') . '</td>');
      $html[] = '</tr>';
      $html[] = '<tr>';
      $html[] = ('<td class="w"><span>' . translate('MSG_WARNING') . '</span></td>');
      $html[] = ('<td> - ' . translate('MSG_DENOTES_TESTS_WITH_WARNINGS') . '</td>');
      $html[] = '</tr>';
      $html[] = '<tr>';
      $html[] = ('<td class="f"><span>' . translate('MSG_FAILED') . '</span></td>');
      $html[] = ('<td> - ' . translate('MSG_DENOTES_FAILED_TESTS') . '</td>');
      $html[] = '</tr>';
      $html[] = '</table>';
    }
    
    $html[] = '<table>';

    if (count($records->nodeset) <= 0)
    {
      $html[] = '<tr class="m">';
      $html[] = ('<td>' . translate('MSG_DIAGNOSIS_LOG') . ' ' . translate('MSG_IS_EMPTY') . '</td>');
      $html[] = '<tr>';
    }
    else
    {
      $html[] = '<thead>';
      $html[] = '<tr>';
      $html[] = ('<td>' . translate('MSG_DEVICE') . ', ' . translate('MSG_TEST') . '</td>');
      $html[] = ('<td class="c">' . translate('MSG_NUMBER_OF_TIMES_DIAGNOSED') . '</td>');
      $html[] = ('<td class="c">' . translate('MSG_FAILED') . ' ' . translate('MSG_COUNT') . '</td>');
      $html[] = ('<td class="c">' . translate('MSG_PASSED_WITH_WARNINGS') . ' ' . translate('MSG_COUNT') . '</td>');
      $html[] = ('<td class="c">' . translate('MSG_DIAGNOSIS_TIME') . '</td>');
      $html[] = '<td>&nbsp;</td>';
      $html[] = '</tr>';
      $html[] = '</thead>';
      $html[] = '<tbody>';

      foreach ($records->nodeset as $record)
      {
        $test = $record->get_attribute('diagnosisCaption');
        $passedCount = $record->get_attribute('passedCount');
        $warningCount = $record->get_attribute('passedWithWarningsCount');
        $failedCount = $record->get_attribute('failedCount');
        $numberOfTests = ($passedCount + $warningCount + $failedCount);
        $time = $record->get_attribute('testTime');
        if ($failedCount > 0)
        {
          $result = 'failed';
          $resultCaption = translate('MSG_FAILED');
          $resultClass = ' class="f"';
        }
        else if ($warningCount > 0)
        {
          $result = 'warned';
          $resultCaption = translate('MSG_WARNING');
          $resultClass = ' class="w"';
        }
        else
        {
          $result = 'passed';
          $resultCaption = translate('MSG_PASSED');
          $resultClass = ' class="p"';
        }

        $html[] = ('<tr>');
        $html[] = ('<td>' . $test . '</td>');
        $html[] = ('<td class="ct">' . $numberOfTests . '</td>');
        $html[] = ('<td class="ct">' . $failedCount . '</td>');
        $html[] = ('<td class="ct">' . $warningCount . '</td>');
        $html[] = ('<td>');
        $html = array_merge($html, buildSaveableLogColumn($record, 'testTime', 'diagnosisResult', 'caption!?'));
        $html[] = ('</td>');
        $html[] = ('<td>');
        $html = array_merge($html, buildSaveableLogColumn($record, 'currentState', 'diagnosisResult', 'caption!?'));
        $html[] = ('</td>');
        $html[] = '</tr>';

        $diagnosisResults = getDiagnosisResults($record);
        foreach ($diagnosisResults as $diagnosisResult)
        {
          generateErrorTableForSave($diagnosisResult, $html);
          $innerResults = getDiagnosisResults($diagnosisResult);
          if (count($innerResults) > 0)
          {
            foreach ($innerResults as $innerResult)
              generateErrorTableForSave($innerResult, $html);
          }
        }
      }
      $html[] = '</tbody>';
    }

    $html[] = '</table>';
  }
  
  function outputSaveableTestLog($forAllLog, &$html)
  {
    // Call AI
    $stdout = AIBridge::getTestLog();
    // Try to make stdout into a XMLDocument
    $xml = new XMLDocument($stdout);
    if ($xml)
    {
      $document = $xml->document_element();
      if ($document->tagname() == 'error') // If AI returned an error:
      {
        $outputArray = array(
          'name'  => 'AI Error',
          'value' => $document->get_content());
        if ($forAllLog == false) die(Rosetta::arrayToJSON($outputArray));
      }
      $xpath = xpath_new_context($document);
      $records = @ xpath_eval($xpath, '/testLogRecord');
    }
    else
    {
      $outputArray = array(
        'name'  => 'XML Error',
        'value' => $stdout);
      if ($forAllLog == false) die(Rosetta::arrayToJSON($outputArray));
    }

    // Output HTML
    if (count($records->nodeset) > 0)
    {
      $html[] = '<table class="i">';
      $html[] = '<tr>';
      $html[] = ('<td class="p"><span>' . translate('MSG_PASSED') . '</span></td>');
      $html[] = ('<td> - ' . translate('MSG_DENOTES_PASSED_TESTS') . '</td>');
      $html[] = '</tr>';
      $html[] = '<tr>';
      $html[] = ('<td class="w"><span>' . translate('MSG_WARNING') . '</span></td>');
      $html[] = ('<td> - ' . translate('MSG_DENOTES_TESTS_WITH_WARNINGS') . '</td>');
      $html[] = '</tr>';
      $html[] = '<tr>';
      $html[] = ('<td class="f"><span>' . translate('MSG_FAILED') . '</span></td>');
      $html[] = ('<td> - ' . translate('MSG_DENOTES_FAILED_TESTS') . '</td>');
      $html[] = '</tr>';
      $html[] = '</table>';
    }

    $html[] = '<table>';

    if (count($records->nodeset) <= 0)
    {
      $html[] = '<tr class="m">';
      $html[] = ('<td>' . translate('MSG_TEST_LOG') . ' ' . translate('MSG_IS_EMPTY') . '</td>');
      $html[] = '<tr>';
    }
    else
    {
      $html[] = '<thead>';
      $html[] = '<tr>';
      $html[] = ('<td>' . translate('MSG_DEVICE') . ', ' . translate('MSG_TEST') . '</td>');
      $html[] = ('<td class="c">' . translate('MSG_NUMBER_OF_TIMES_TESTED') . '</td>');
      $html[] = ('<td class="c">' . translate('MSG_NUMBER_OF_ERRORS') . '</td>');
      $html[] = ('<td class="c">' . translate('MSG_TEST_TIME') . '</td>');
      $html[] = ('<td class="c">' . translate('MSG_LAST_COMPLETION') . '</td>');
      $html[] = '<td>&nbsp;</td>';
      $html[] = '</tr>';
      $html[] = '</thead>';
      $html[] = '<tbody>';

      foreach ($records->nodeset as $record)
      {
        $test = $record->get_attribute('testCaption');
        $passedCount = $record->get_attribute('passedCount');
        $failedCount = $record->get_attribute('failedCount');
        $time = $record->get_attribute('testTime');

        $xpath = xpath_new_context($record);
        $completions = @ xpath_eval($xpath, '/testCompletion[@result!=""]'); // only get test completions with actual results
        $testCount = $passedCount + $failedCount;
        $completionCount = count($completions->nodeset);
        
        if ($completionCount)
        {
          $lastCompletion = $completions->nodeset[$completionCount-1]; // get the last testCompetion
          $lastCompletionTime = $lastCompletion->get_attribute('completionTime');
          $result = $lastCompletion->get_attribute('result');
        }
        else
        {
          $lastCompletionTime = '';
          $result = '';
        }
        $resultCaption = ($result == 'passed') ? translate('MSG_PASSED') : (($result == 'failed') ? translate('MSG_FAILED') : '');
        $resultClass = ($result == 'passed') ? ' class="p"' : (($result == 'failed') ? ' class="f"' : '');

        $html[] = ('<tr>');
        $html[] = ('<td>' . $test . '</td>');
        $html[] = ('<td class="ct">' . $testCount . '</td>');
        $html[] = ('<td class="ct">' . $failedCount . '</td>');
        $html[] = ('<td>');
        $html = array_merge($html, buildSaveableLogColumn($record, 'testTime', 'testCompletion', 'caption!?'));
        $html[] = ('</td>');
        $html[] = ('<td>');
        $html = array_merge($html, buildSaveableLogColumn($record, 'completionTime', 'testCompletion', 'caption!?'));
        $html[] = ('</td>');
        $html[] = ('<td>');
        $html = array_merge($html, buildSaveableLogColumn($record, 'result', 'testCompletion', 'caption!?'));
        $html[] = ('</td>');
        $html[] = '</tr>';
      }
      $html[] = '</tbody>';
    }
    $html[] = '</table>';
  }
  
  function outputSaveableErrorLog($forAllLog, &$html)
  {
    // Call AI
    $stdout = AIBridge::getErrorLog();

    // Try to make stdout into a XMLDocument
    $xml = new XMLDocument($stdout);
    if ($xml)
    {
      $document = $xml->document_element();
      if ($document->tagname() == 'error') // If AI returned an error:
      {
        $outputArray = array(
          'name'  => 'AI Error',
          'value' => $document->get_content());
        if ($forAllLog == false) die(Rosetta::arrayToJSON($outputArray));
      }
      $xpath = xpath_new_context($document);
      $records = @ xpath_eval($xpath, '/errorLogRecord');
    }
    else
    {
      $outputArray = array(
        'name'  => 'XML Error',
        'value' => $stdout);
      if ($forAllLog == false) die(Rosetta::arrayToJSON($outputArray));
    }

    // Output HTML
    $html[] = '<table>';

    if (count($records->nodeset) <= 0)
    {
      $html[] = '<tr class="m">';
      $html[] = ('<td>' . translate('MSG_ERROR_LOG') . ' ' . translate('MSG_IS_EMPTY') . '</td>');
      $html[] = '</tr>';
    }
    else
    {
      $html[] = '<tbody>';

      foreach ($records->nodeset as $record)
      {
        $test = $record->get_attribute('testCaption');
        $deviceLocation = $record->get_attribute('deviceLocation');
        $deviceLocation = str_replace( "\n", "<br>", $deviceLocation );

        $xpath = xpath_new_context($record);
        $errors = @ xpath_eval($xpath, '/error');
        foreach ($errors->nodeset as $error)
        {
          $caption = $error->get_attribute('caption');
          $failedCount = $error->get_attribute('count');
          $errorCode = $error->get_attribute('errorCode');
          $errorContent = $error->get_content();

          $repairContent = '';
          $xpath = xpath_new_context($error);
          $repairActions = @ xpath_eval($xpath, '/repairAction');
          $numberOfRepairActions = count($repairActions->nodeset);
          if ($numberOfRepairActions)
          { 
            $repairAction = $repairActions->nodeset[0]; // there should only be 1, if there are more, we ignore the remaining
            $repairContent = $repairAction->get_content();
            $errorContent = substr($errorContent, 0, strpos($errorContent, $repairContent)); // remove the repair action from the content
          }

          if ($errorContent == '')
            $description = $caption;
          else
            $description = ($caption . '<br>' . $errorContent);

          $description = str_replace( "\n", "<br>", $description );
          $repairContent = str_replace( "\n", "<br>", $repairContent );
          
          $html[] = '<tr class="r1">';
          $html[] = ('<th class="t">' . translate('MSG_DEVICE') . ', ' . translate('MSG_TEST') . ':</th>');
          $html[] = ('<td class="t">' . $test . '</td>');
          $html[] = ('<th class="n">' . translate('MSG_NUMBER_OF_ERRORS') . ':</th>');
          $html[] = ('<td class="n">' . $failedCount . '</td>');
          $html[] = ('<th class="c">' . translate('MSG_ERROR_CODE') . ':</th>');
          $html[] = ('<td class="c">' . $errorCode . '</td>');
          $html[] = '</tr>';

          $html[] = ('<tr><th colspan="6">' . translate('MSG_DESCRIPTION') . ':</th></tr>');
          $html[] = ('<tr><td colspan="6">' . $description . '</td></tr>');

          if ($repairContent != '') {
            $html[] = ('<tr><th colspan="6">' . translate('MSG_REPAIR') . ':</th></tr>');
            $html[] = ('<tr><td colspan="6">' . $repairContent . '</td></tr>');
          }

          if ($deviceLocation != '') {
            $html[] = ('<tr><th colspan="6">' . translate('MSG_DEVICE_LOCATION') . ':</th></tr>');
            $html[] = ('<tr><td colspan="6">' . $deviceLocation . '</td></tr>');
          }
        }
      }
      $html[] = '</tbody>';
    }

    $html[] = '</table>';
  }

  function outputSaveableIMLLog($sortField, $forAllLog, &$html)
  {
    // Call AI
    $imlLog = AIBridge::getImlLog($sortField);

    // Try to make stdout into a XMLDocument
    $xml = new XMLDocument($imlLog);
    if ($xml)
    {
      $document = $xml->document_element();
      if ($document->tagname() == 'error') // If AI returned an error:
      {
        $outputArray = array(
          'name'  => 'AI Error',
          'value' => $document->get_content());
        if ($forAllLog == false) die(Rosetta::arrayToJSON($outputArray));
      }
      $xpath = xpath_new_context($document);
      $structures = @ xpath_eval($xpath, '/structure');
    }
    else
    {
      $outputArray = array(
        'name'  => 'XML Error',
        'value' => $stdout);
      if ($forAllLog == false) die(Rosetta::arrayToJSON($outputArray));
    }

    $html[] = '<table>';

    if (count($structures->nodeset) <= 0)
    {
      $root = $xml->document_element();
      if (strstr($root->get_attribute('caption'), translate('MSG_NOT_AVAILABLE')) === FALSE)
      {
        $html[] = '<tr class="m">';
        $html[] = ('<td>' . translate('MSG_INTEGRATED_MANAGEMENT_LOG') . ' ' . translate('MSG_IS_EMPTY') . '</td>');
        $html[] = '</tr>';
      }
      else
      {
        $html[] = '<tr class="m">';
        $html[] = ('<td>' . translate('MSG_INTEGRATED_MANAGEMENT_LOG') . ': ' . translate('MSG_NOT_AVAILABLE') . '</td>');
        $html[] = '</tr>';
      }
    }
    else
    {
      $html[] = '<thead>';
      $html[] = '<tr>';
      $html[] = ('<td>' . translate('MSG_SEVERITY') . '</th>');
      $html[] = ('<td class="c">' . translate('MSG_CLASS') . '</th>');
      $html[] = ('<td class="c">' . translate('MSG_LAST_UPDATE') . '</th>');
      $html[] = ('<td class="c">' . translate('MSG_INITIAL_UPDATE') . '</th>');
      $html[] = ('<td class="c">' . translate('MSG_COUNT') . '</th>');
      $html[] = ('<td>' . translate('MSG_DESCRIPTION') . '</th>');
      $html[] = '</tr>';
      $html[] = '</thead>';
      $html[] = '<tbody>';

      foreach ($structures->nodeset as $structure)
      {
        $structureXpath = xpath_new_context($structure);
        $properties = @ xpath_eval($structureXpath, '/property');

        $class = $structure->get_attribute('class');
        $code = $structure->get_attribute('code');
        $index = $structure->get_attribute('index');

        $description = $code;

        $individualRows = array('severity', 'count', 'lastupdate', 'initialupdate', 'message');
        $rowParameters = array();
        foreach ($properties->nodeset as $property)
        {
          $name = strtolower($property->get_attribute('name'));
          $value = $property->get_attribute('value');
          if (in_array($name, $individualRows))
          {
            $rowParameters[$name] = $value;
          }
          else
          {
            $caption = $property->get_attribute("caption");
            $description .= (' - ' . $caption . ': ' . $value);
          }
        }
        
        $html[] = '<tr>';
        $html[] = ('<td>' . $rowParameters['severity'] . '</td>');
        $html[] = ('<td class="c">' . $class . '</td>');
        $html[] = ('<td class="c">' . $rowParameters['lastupdate'] . '</td>');
        $html[] = ('<td class="c">' . $rowParameters['initialupdate'] . '</td>');
        $html[] = ('<td class="c">' . $rowParameters['count'] . '</td>');
        $html[] = ('<td class="c">' . $rowParameters['message'] . '</td>');
        $html[] = '</tr>';
      }
      $html[] = '</tbody>';
    }

    $html[] = '</table>';
    
  }

  function outputSaveableSELLog($sortField, $forAllLog, &$html)
  {
    // Call AI
    $selLog = AIBridge::getSelLog($sortField);

    // Try to make stdout into a XMLDocument
    $xml = new XMLDocument($selLog);
    if ($xml)
    {
      $document = $xml->document_element();
      if ($document->tagname() == 'error') // If AI returned an error:
      {
        $outputArray = array(
          'name'  => 'AI Error',
          'value' => $document->get_content());
        if ($forAllLog == false) die(Rosetta::arrayToJSON($outputArray));
      }
      $xpath = xpath_new_context($document);
      $structures = @ xpath_eval($xpath, '/structure');
    }
    else
    {
      $outputArray = array(
        'name'  => 'XML Error',
        'value' => $stdout);
      if ($forAllLog == false) die(Rosetta::arrayToJSON($outputArray));
    }

    $html[] = '<table>';

    if (count($structures->nodeset) <= 0)
    {
      $root = $xml->document_element();
      if (strstr($root->get_attribute('caption'), translate('MSG_NOT_AVAILABLE')) === FALSE)
      {
        $html[] = '<tr class="m">';
        $html[] = ('<td>' . translate('MSG_SYSTEM_EVENT_LOG') . ' ' . translate('MSG_IS_EMPTY') . '</td>');
        $html[] = '</tr>';
      }
      else
      {
        $html[] = '<tr class="m">';
        $html[] = ('<td>' . translate('MSG_SYSTEM_EVENT_LOG') . ': ' . translate('MSG_NOT_AVAILABLE') . '</td>');
        $html[] = '</tr>';
      }
    }
    else
    {
      $html[] = '<thead>';
      $html[] = '<tr>';
      $html[] = ('<td>' .translate('MSG_SENSOR_NAME') . '</th>');
      $html[] = ('<td class="c">' . translate('MSG_SENSOR_TYPE') . '</th>');
      $html[] = ('<td class="c">' . translate('MSG_DESCRIPTION') . '</th>');
      $html[] = ('<td class="c">' . translate('MSG_INITIAL_OCCURRENCE') . '</th>');
      $html[] = ('<td class="c">' . translate('MSG_LAST_OCCURRENCE') . '</th>');
      $html[] = ('<td>' . translate('MSG_COUNT') . '</th>');
      $html[] = '</tr>';
      $html[] = '</thead>';
      $html[] = '<tbody>';

      foreach ($structures->nodeset as $structure)
      {
        $structureXpath = xpath_new_context($structure);
        $properties = @ xpath_eval($structureXpath, '/property');

        $index = $structure->get_attribute('index');

        $individualRows = array('sensorName','sensorType','description','initialOccurrence','lastOccurrence', 'count');
        $rowParameters = array();
        foreach ($properties->nodeset as $property)
        {
          $name = $property->get_attribute('name');
          $value = $property->get_attribute('value');
          if (in_array($name, $individualRows))
          {
            $rowParameters[$name] = $value;
          }
          else
          {
            $caption = $property->get_attribute("caption");
            $description .= (' - ' . $caption . ': ' . $value);
          }
        }

        $html[] = '<tr>';
        $html[] = ('<td>' . $rowParameters['sensorName'] . '</td>');
        $html[] = ('<td class="c">' . $rowParameters['sensorType'] . '</td>');
        $html[] = ('<td class="c">' . $rowParameters['description'] . '</td>');
        $html[] = ('<td class="c">' . $rowParameters['initialOccurrence'] . '</td>');
        $html[] = ('<td class="c">' . $rowParameters['lastOccurrence'] . '</td>');
        $html[] = ('<td class="c">' . $rowParameters['count'] . '</td>');
        $html[] = '</tr>';
      }
      $html[] = '</tbody>';
    }

    $html[] = '</table>';    
  }
  
}

?>
