/********************************************************************
//        NAME- ProgressIndicator
// DESCRIPTION- Constructor for the ProgressIndicator object
//      PASSED- element (HTMLElement / string): optional argument,
//                  if provided, the main element of the progress indicator
//                  will be set to this, otherwise a blank span will
//                  be created to serve as the main element
//     RETURNS- 
//       NOTES- The ProgressIndicator class controls the appearance and 
//              adjustment of a progress indicator on screen
********************************************************************/
function ProgressIndicator(element)
{
  this.percent = 0;
  
  if (element == undefined)
    this.element = newElement('span');
  else if (isElement(element))
    this.element = element;
  else if (typeof element == 'string')
    this.element = getElement(element);
  else
    throw('ProgressIndicator: invalid element - '+element);

  this.init();
}

/********************************************************************
//        NAME- ProgressIndicator.prototype.init
// DESCRIPTION- Initializes the element of the progress indicator
//      PASSED- 
//     RETURNS- 
//       NOTES- 
********************************************************************/
ProgressIndicator.prototype.init = function init()
{
  addClass(this.element, 'progressIndicator');
  this.element.controller = this;
  this.element.setPercent = function (percent)
  {
    return this.controller.setPercent(percent)
  };
}

/********************************************************************
//        NAME- ProgressIndicator.prototype.setPercent
// DESCRIPTION- Sets the percentage of the progress indicator
//      PASSED- percent (number): the percentage to set the progress
//                  indicator to. Should be an integer between 0 and 100
//     RETURNS- 
//       NOTES- percent will be trimmed to an int between 0 and 100
//              if necessary. Upon 100%, progress indicator adds
//              'ui_progressIndicatorFinished' to the stretcher's className
********************************************************************/
ProgressIndicator.prototype.setPercent = function setPercent(percent, className)
{
  if (percent != Number(percent))
    throw('ProgressIndicator: invalid percent in ProgressIndicator.setPercent() - '+percent);
  if (percent == undefined || percent == '' || percent < 0)
    percent = 0;
  if (percent > 100)
    percent = 100;

  this.percent = Math.round(percent);
  clearChildren(this.element);
  this.element.appendChild(newText(this.percent+'%'));
  return this;
}
